@@ -12,24 +12,18 @@ import datetime
 import logging
 import os
 import sys
+import textwrap
 
 import pynsot
 from . import client
 from .util import get_result
 
-from .vendor import click
-from .vendor import prettytable
+from .vendor import click, netaddr, prettytable
 from .vendor.slumber.exceptions import (HttpClientError, HttpServerError)
 
 
-__author__ = 'Jathan McCollum'
-__maintainer__ = 'Jathan McCollum'
-__email__ = 'jathan@dropbox.com'
-__copyright__ = 'Copyright (c) 2015 Dropbox, Inc.'
-
-
 # Constants/Globals
-if os.getenv('DEBUG'):
+if os.getenv('PYNSOT_DEBUG'):
     logging.basicConfig(level=logging.DEBUG)
 log = logging.getLogger(__name__)
 
@@ -61,8 +55,11 @@ NATURAL_KEYS = {
 GREP_FORMATS = {
     'devices': '%(hostname)s',
     'networks': '%(network_address)s/%(prefix_length)s',
-    'attributes': '%(name)s:%(resource_name)s',
+    'addresses': '%(network_address)s/%(prefix_length)s',  # Same as networks
+    'assignments': '%(hostname)s:%(interface_name)s:%(address)s',
+    'attributes': '%(resource_name)s:%(name)s',
     'interfaces': '%(device)s:%(name)s',
+    'sites': '%(name)s',
 }
 
 
@@ -107,10 +104,15 @@ class App(object):
     def __init__(self, ctx, client_args=None, verbose=False):
         if client_args is None:
             client_args = {}
+
+        # Force api_version 1.0 for CLI util.
+        client_args['extra_args'] = {'api_version': '1.0'}
+
         self.client_args = client_args
         self.ctx = ctx
         self.verbose = verbose
         self.resource_name = self.ctx.invoked_subcommand
+        self.grep_name = self.resource_name
         self.site_id = None  # This is populated later.
         self.rebase_done = False  # So that we only rebase once.
 
@@ -229,7 +231,7 @@ class App(object):
             msg = str(err)
 
         if isinstance(msg, dict):
-            msg = self.pretty_dict(msg, delim=':', joiner='')
+            msg = self.pretty_dict(msg, delim=': ', joiner='')
 
         # If we're being verbose, print some extra context.
         if self.verbose:
@@ -302,6 +304,10 @@ class App(object):
         if field == 'user':
             field_data = field_data['email']
 
+        # Arbitrarily wrap description field at 20 chars
+        if field == 'description':
+            field_data = '\n'.join(textwrap.wrap(field_data, 20))
+
         # If the field is a dict, pretty_dict it!
         if isinstance(field_data, dict):
             # If this is an inner dict, prettify it, too.
@@ -335,7 +341,10 @@ class App(object):
         :param obj:
             Object
         """
-        return GREP_FORMATS[self.resource_name] % obj
+        # If there's a "grep_name", use that, otherwise use resource_name
+        grep_name = getattr(self, 'grep_name', self.resource_name)
+
+        return GREP_FORMATS[grep_name] % obj
 
     def print_grep(self, objects):
         """
@@ -349,13 +358,38 @@ class App(object):
         output = []
         for obj in objects:
             prefix = self.format_object_for_grep(obj)
-            attrs = obj['attributes']
+            attrs = obj.get('attributes', {})
             keys = sorted(attrs)
             for k in keys:
                 output.append('%s %s=%s' % (prefix, k, attrs[k]))
 
         click.echo('\n'.join(output))
 
+    def print_by_natural_key(self, objects, delimiter='\n'):
+        """
+        Print a list of objects by their natural_key
+
+        :param objects:
+            List of objects
+
+        :param delimiter:
+            Character used to delimit objects
+        """
+        output = []
+        for obj in objects:
+            output.append(self.format_object_for_grep(obj))
+
+        # Networks results must be specially sorted.
+        if self.grep_name != 'networks':
+            output = sorted(output)
+        else:
+            networks = sorted(
+                netaddr.IPNetwork(d) for d in output
+            )
+            output = (str(n) for n in networks)
+
+        click.echo(delimiter.join(output))
+
     def print_list(self, objects, display_fields):
         """
         Print a list of objects in a table format.
@@ -393,7 +427,7 @@ class App(object):
         table = prettytable.PrettyTable(headers)
 
         # Display table in a frame
-        table.vrules = prettytable.FRAME
+        table.vrules = prettytable.prettytable.FRAME
 
         # *or* Display table with row separators
         # table.hrules = prettytable.ALL
@@ -481,7 +515,12 @@ class App(object):
         # Construct our params to retrieve a single object.
         params = {'limit': 1}
         for natural_key in natural_keys:
-            params[natural_key] = data[natural_key]
+            natural_value = data.get(natural_key)
+
+            # Only filter if the key was found. This is mostly for interfaces
+            # where (device_id,) or (device_id, name) are valid lookups.
+            if natural_value is not None:
+                params[natural_key] = natural_value
 
         # Rebase before we toilet face.
         if resource is None:
@@ -495,9 +534,9 @@ class App(object):
 
         # Assert only 1 matching result.
         try:
-            total = r['data']['total']
-        except KeyError:
             total = r['count']
+        except KeyError:
+            return r  # Single result such as from /api/networks/:id/parent/
 
         if total > 1:
             log.debug('get_single_object: More than one object found!')
@@ -505,22 +544,48 @@ class App(object):
 
         # Return a single result.
         r = get_result(r)
-        r = self.get_paginated_results(r)
         try:
             return r[0]
         except IndexError:
             return None
 
-    def get_paginated_results(self, objects):
+    def detail(self, data, resource):
         """
-        If ``objects`` has a results key, return that instead.
+        GET a detail endpoint and return the results.
 
-        :param objects:
-            List or dict of objects.
+        This method is currently only called for Network.next_address() and
+        Network.next_network(), so its use-case is very specific.
+
+        :param data:
+            Dict of query parameters
+
+        :param resource:
+            A prepared API resource object
         """
-        if 'results' in objects:
-            return objects['results']
-        return objects
+        try:
+            return resource.get(**data)
+        except HTTP_ERRORS as err:
+            self.handle_error('detail', data, err)
+
+    def set_query(self, data, delimited=False):
+        """
+        Run a set query and return the results.
+
+        :param data:
+            Dict of query parameters
+
+        :param delimited:
+            Whether to display the results as comma or newline delimited
+        """
+        self.rebase(data)
+        try:
+            results = self.resource.query.get(**data)
+        except HTTP_ERRORS as err:
+            self.handle_error('list', data, err)
+
+        objects = get_result(results)
+        delimiter = ',' if delimited else '\n'
+        self.print_by_natural_key(objects, delimiter)
 
     def list(self, data, display_fields=None, resource=None,
              verbose_fields=None):
@@ -545,6 +610,7 @@ class App(object):
         obj_id = data.get('id')  # If obj_id, it's a single object
 
         grep = data.pop('grep', False)
+        by_natural_key = data.pop('natural_key', False)
 
         # If a resource object is provided, call it instead, and only rebase if
         # we haven't provided our own resource.
@@ -558,7 +624,7 @@ class App(object):
             if obj_id:
                 log.debug('Retrieving by obj_id=%r' % obj_id)
                 result = resource(obj_id).get()
-                obj = result['data'][self.singular]
+                obj = get_result(result)
 
             # If we still don't have an object try param-based lookup.
             if obj is None:
@@ -566,8 +632,6 @@ class App(object):
                 obj = self.get_single_object(data, resource)
 
             # If obj is STILL None...
-            # Or get all of them.
-            # else:
             if obj is not None:
                 # Set display_fields to verbose
                 display_fields = verbose_fields or display_fields
@@ -587,14 +651,13 @@ class App(object):
 
             # Or just list all of them.
             elif result:
-                # FIXME(jathan): Once API is fully-versioned, deprecate this
-                # call.
                 objects = get_result(result)
 
             if objects:
-                objects = self.get_paginated_results(objects)
                 if grep:
                     self.print_grep(objects)
+                elif by_natural_key:
+                    self.print_by_natural_key(objects)
                 else:
                     self.print_list(objects, display_fields)
             else:
@@ -698,7 +761,7 @@ class App(object):
             if obj_id:
                 log.debug('Retrieving by obj_id=%r' % obj_id)
                 result = self.resource(obj_id).get()
-                obj = result['data'][self.singular]
+                obj = get_result(result)
             else:
                 obj = self.get_single_object(data)
         except HTTP_ERRORS as err:
@@ -749,7 +812,7 @@ def app(ctx, verbose):
     """
     Network Source of Truth (NSoT) command-line utility.
 
-    For detailed documentation, please visit https://nsot.readthedocs.org
+    For detailed documentation, please visit https://nsot.readthedocs.io
     """
     # This is the "app" object attached to all contexts.
     ctx.obj = App(ctx=ctx, verbose=verbose)
